import { getEnvGitHubToken } from "@/lib/app/env";
import { Octokit } from "octokit";
import type { GitHubRequest } from "../type";
import { githubErrorHasStatus } from "./blog/error";
import { getGitHubCache, setGitHubCache } from "./cache";
import type { GitHubContent } from "./type";

const _octokit: { current: Octokit | null } = { current: null };

const getOctokit = (): Octokit => {
	if (_octokit.current === null) {
		_octokit.current = new Octokit({
			// Doesn't really need "auth" but provide to have better rate limit
			auth: getEnvGitHubToken(),
		});
	}
	return _octokit.current;
};

export const fetchGitHubContent = async (
	request: GitHubRequest
): Promise<GitHubContent> => {
	const { owner, path, repo } = request;
	const cached = await getGitHubCache(request);
	try {
		const response = await getOctokit().rest.repos.getContent({
			...{ owner, path, repo },
			headers: { "If-None-Match": cached?.headers.etag },
			mediaType: { format: "json" },
		});
		setGitHubCache(request, response);
		return response.data;
	} catch (error) {
		// Return cache
		if (!githubErrorHasStatus(error)) throw error;
		if (error.status !== 304) throw error;
		if (cached === null) throw Error("No cache but 304");
		return cached.data;
	}
};
